function [call,nu,eta,d1] = Longstaff_Schwartz(x,v,T,K,r,paramLS)
%	
% Nabil Tahani, HEC Montral. Copyright 2003.
%
% This function computes Credit spread options under 
% Longstaff & Schwartz (1995) model
%	

mu      = paramLS(1); % drift of the log-spread
alpha   = paramLS(2); % mean-reverting coeff of the log-spread
p       = paramLS(3); % drift of the risk free rate
q       = paramLS(4); % mean-reverting coeff of the risk free rate
sig_r   = paramLS(5); % vol of the risk free rate
rho_Xr  = paramLS(6); % correlation between the risk free rate and the log-spread
s       = sqrt(v);    % vol of the log-spread
%mu      = r - .5*s^2; % Black&Scholes adjustment

% Parameters used
if q~=0 & alpha~=0
   nu = x*exp(-alpha*T) + (1/alpha)*(mu-(rho_Xr*sig_r*s)/q)*(1-exp(-alpha*T))...
    						   	  + rho_Xr*sig_r*s*(1-exp(-(alpha+q)*T))/(q*(alpha+q));
elseif q==0 & alpha~=0  
   nu = x*exp(-alpha*T) + (mu/alpha)*(1-exp(-alpha*T)); 
elseif q~=0 & alpha==0
   nu = x + (mu-(rho_Xr*sig_r*s)/q)*T...
      + rho_Xr*sig_r*s*(1-exp(-(alpha+q)*T))/(q*(alpha+q));
elseif q==0 & alpha==0   
   nu = x + mu*T;
end;

if alpha~=0 eta = sqrt(s^2*(1-exp(-2*alpha*T))/(2*alpha));
else eta = sqrt(s^2*T);
end;
   
d1 = (1./eta).*(-log(K) + nu + eta.^2);

% Discount factor
D = exp(-r*T);

% Call value
call = D.*exp(nu+0.5*eta.^2).*normcdf(d1) - K*D.*normcdf(d1-eta);
