function [L,L_1,R,W,WeR] = laguerre_poly(n)
%	
% Nabil Tahani, HEC Montral. Copyright 2003.
%
% This function computes the n th and (n-1)th Laguerre polynomials
% and gives the roots of the n th polynomial. These roots serve to
% evaluate the weights needed for Gaussian-quadrature integration
%
%		L	 	: the n th Laguerre polynomial
%		L_1 	: the (n-1)th Laguerre polynomial
%		R		: the roots of the n th Laguerre polynomial
%		W 		: the weights needed to compute integrals of exp(-x)f(x) over [0,infinity[
%		WeR	: the weights needed to compute integrals of f(x) over [0,infinity[
%

format long e;
P = zeros(n+2,n+1);
P(2,1) = 1;
P(2,2) = 1;
P(3,2) =-1;
R = zeros(n);

% Loop to compute the first n laguerre polynomials coefficients 
for i = 2:n
   P(2:n+2,i+1) = ((2*i-1)*P(2:n+2,i)-P(1:n+1,i)-(i-1)*P(2:n+2,i-1))/i;
end;

L = P(n+2:-1:2,n+1);
if n>0
   L_1 = P(n+1:-1:2,n); 
   R = roots(L);
   R = sort(R);
	W = (1/n^2)*R./(Polyval(L_1,R).^2);
	WeR = W.*exp(R);
else L_1 = 'Indefined'; W = 'Indefined'; WeR = 'Indefined'; 
end;
