function [out1,out2,out3] = ou_ode(t,y,flag,tau,psi,param)
%	
% Nabil Tahani, HEC Montral. Copyright 2003.
%
% This function solves the ODEs under the O-U mean reverting model
%

mu     = param(1);
alpha  = param(2);
gamma  = param(3);
rho    = param(4);
sigma  = param(5);
lambda = param(6);
kappa  = param(7);
theta  = param(8);

[e1,e2,e3] = e(t,gamma,alpha,rho,sigma,lambda,kappa,theta,psi,'1');

tspan = tau;
init_cond = [2*e3 ; 0 ; 0];
tol = odeset('RelTol',1e-12,'AbsTol',1e-12);

if nargin < 3 | isempty(flag) 
   out1 = [sigma^2*y(1)^2-2*lambda*y(1)-2*e1;
   		  sigma^2*y(1)*y(2)-lambda*y(2)+kappa*theta*y(1)-e2;
          (1/2)*sigma^2*(y(1)+y(2)^2)+kappa*theta*y(2)]; 
else
  switch(flag)
    case 'init'               % Return tspan, y0, and options
       out1 = tspan;
       out2 = init_cond;
       out3 = tol;
    otherwise
       error(['Unknown request ''' flag '''.']);
    end
end
 
 
%*************************************************************** 

function [e1,e2,e3] = e(t,gamma,alpha,rho,sigma,lambda,kappa,theta,psi,risk_type)
switch (risk_type)
   case '1' % if the log-spread risk premium is proportional to vol^2
		e1 = gamma*psi*exp(-alpha*t)-(1/2)*(1-rho^2)*psi^2*exp(-2*alpha*t)...
		   							    +(rho/sigma)*(0.5*alpha-lambda)*psi*exp(-alpha*t); 
		e2 = (rho*kappa*theta/sigma)*psi*exp(-alpha*t);                           
      e3 = rho*psi/(2*sigma);
   case '2' % if the log-spread risk premium is proportional to vol
      e1 = -(1/2)*(1-rho^2)*psi^2*exp(-2*alpha*t)+rho*psi*(alpha/2-lambda)*exp(-alpha*t)/sigma; 
		e2 = (rho*kappa*theta/sigma+gamma)*psi*exp(-alpha*t);                           
      e3 = rho*psi/(2*sigma);
	end

