function [out1,out2,out3] = sqroot_ode(t,y,flag,tau,psi,param)
%	
% Nabil Tahani, HEC Montral. Copyright 2003.
%
% This function solves the ODEs under the square-root mean reverting model
%

mu     = param(1);
alpha  = param(2);
gamma  = param(3);
rho    = param(4);
sigma  = param(5);
lambda = param(6);
kappa  = param(7);
theta  = param(8);

[e1,e2] = e(t,gamma,alpha,rho,sigma,lambda,kappa,theta,psi);

tspan = tau;
init_cond = [e2 ; 0];
tol = odeset('RelTol',1e-012,'AbsTol',1e-012);

if nargin < 3 | isempty(flag) 
   out1 = [.5*sigma^2*y(1)^2-lambda*y(1)- e1;
           kappa*theta*y(1)]; 
else
  switch(flag)
    case 'init'               % Return tspan, y0, and options
       out1 = tspan;
       out2 = init_cond;
       out3 = tol;
    otherwise
       error(['Unknown request ''' flag '''.']);
    end
end


%*************************************************************** 

function [e1,e2] = e(t,gamma,alpha,rho,sigma,lambda,kappa,theta,psi)
e1 = gamma*psi*exp(-alpha*t)-(1/2)*(1-rho^2)*psi^2*exp(-2*alpha*t)...
		   						 +(rho*psi*(alpha-lambda)/sigma)*exp(-alpha*t); 
e2 = rho*psi/sigma;
