function [cf,t,y] = sqroot_ou_cf(psi,x,v,tau,param,label)
%	
% Nabil Tahani, HEC Montral. Copyright 2003.
%
% This function computes the square-root and the O-U characteristic functions
% under our mean reverting frameworks
%

mu     = param(1);
alpha  = param(2);
gamma  = param(3);
rho    = param(4);
sigma  = param(5);
lambda = param(6);
kappa  = param(7);
theta  = param(8);

switch(label)
	case 'sqroot'
      [t,y] = ode113('sqroot_ode',[],[],[],tau,psi,param);
      d = y(:,1);
      c = y(:,2);
      if alpha ~= 0
	      cf = exp(psi*exp(-alpha*tau)*x + mu*psi*(1-exp(-alpha*tau))/alpha ...
   			      - rho*psi*exp(-alpha*tau)*v/sigma ...
         			- rho*kappa*theta*psi*(1-exp(-alpha*tau))/(sigma*alpha)) ...
                  .* exp(d*v + c);
      else 
         cf = exp(psi*x + mu*psi*tau ...
   			      - rho*psi*v/sigma ...
         			- rho*kappa*theta*psi*tau/sigma) ...
                  .* exp(d*v + c);         
      end;
            
   case 'ou'
      [t,y] = ode113('ou_ode',[],[],[],tau,psi,param);
      e = y(:,1);
      d = y(:,2);
      c = y(:,3);
      if alpha ~= 0
	      cf = exp(psi*exp(-alpha*tau)*x + mu*psi*(1-exp(-alpha*tau))/alpha ...
   			      - rho*psi*exp(-alpha*tau)*v/(2*sigma) ...
         			- rho*sigma*psi*(1-exp(-alpha*tau))/(2*alpha)) ...
                  .* exp(0.5*e*v + d*sqrt(v) + c);
      else 
			cf = exp(psi*x + mu*psi*tau ...
   			      - rho*psi*v/(2*sigma) ...
         			- rho*sigma*psi*tau/2) ...
                  .* exp(0.5*e*v + d*sqrt(v) + c);
      end;
	end;      

