function [cf] = zhu_sqroot_ou_cf(psi,x,v,tau,param,label)
%	
% Nabil Tahani, HEC Montral. Copyright 2003.
%
% This function computes the square-root and the O-U characteristic functions
% under Zhu (2000) models
%

mu     = param(1);
alpha  = param(2);
gamma  = param(3);
rho    = param(4);
sigma  = param(5);
lambda = param(6);
kappa  = param(7);
theta  = param(8);
 
switch(label)
   case 'sqroot'
		s1 = -psi.*(-0.5+rho*lambda/sigma+0.5*psi*(1-rho^2));
		s2 = rho*psi/sigma;
		gam1 = sqrt(lambda^2+2*sigma^2*s1);
		gam2 = 2*gam1.*exp(-gam1*tau)+(lambda+gam1-sigma^2*s2).*(1-exp(-gam1*tau));
      d =(-(1-exp(-gam1*tau)).*(2*s1+lambda*s2)+gam1.*s2.*(1+exp(-gam1*tau)))./gam2;
      c = 2*(kappa*theta/sigma^2)*log(2*gam1.*exp((lambda-gam1)*tau/2)./gam2);
      
      cf = exp(psi*x + mu*psi*tau ...
   			      - rho*psi*v/sigma ...
         			- rho*kappa*theta*psi*tau/sigma) ...
                  .* exp(d*v + c);
       
   case 'ou'      
		s1 = -psi.*(-0.5+rho*lambda/sigma+0.5*psi*(1-rho^2));
      s2 = rho*kappa*theta*psi/sigma;
      s3 = rho*psi/(2*sigma);
      gam1 = sqrt(lambda^2+2*sigma^2*s1);
      gam2 = (lambda-2*sigma^2*s3)./gam1;
      gam3 = lambda*kappa*theta-s2*sigma^2;
      gam4 = cosh(gam1*tau)+gam2.*sinh(gam1*tau);
      e = lambda/sigma^2-gam1.*(sinh(gam1*tau)+gam2.*cosh(gam1*tau))./(gam4*sigma^2);
      d = ((kappa*theta*gam1-gam2.*gam3).*(1-cosh(gam1*tau))...
         		-(kappa*theta*gam1.*gam2-gam3).*sinh(gam1*tau))./(gam1.*gam4*sigma^2);
      c = -0.5*log(gam4)+(((kappa*theta*gam1-gam2.*gam3).^2 ...
         		-gam3.^2.*(1-gam2.^2)).*sinh(gam1*tau))./(2*gam1.^3.*gam4*sigma^2)...
               +((kappa*theta*gam1-gam2.*gam3).*gam3.*(gam4-1))./(gam1.^3.*gam4*sigma^2)...
               +tau*(gam1.^2*(lambda*sigma^2-(kappa*theta)^2)+gam3.^2)/(2*gam1.^2*sigma^2);
            
      cf = exp(psi*x + mu*psi*tau ...
   			      - rho*psi*v/(2*sigma) ...
         			- rho*sigma*psi*tau/2) ...
                  .* exp(0.5*e*v + d*sqrt(v) + c);
	end   